#ifndef PRIMAX_SCAN_H
#define PRIMAX_SCAN_H

#ifndef TRUE
#define TRUE    1
#endif
#ifndef FALSE
#define FALSE   0
#endif            

#define PORT 0x378;

/* change this to FALSE and you get slow movements*/
#define USE_SCHEDULER TRUE

/* This is a handy macro for debugging */
#define DBG(level, msg, args...)  \
  do {                            \
    if ((dbg_level) & (level))    \
      printf (msg, ##args);       \
  } while (0)

#define CHECK(val) \
  (((val) > 255 ) ? (255) : (val))

/* differen debug level */
#define DBG_LOW                       0x01
#define DBG_MEDIUM                    0x02
#define DBG_HIGH                      0x04
#define DBG_HELPERS                   0x08
#define DBG_TIMEOUT                   0x10
#define DBG_SCAN                      0x20
#define DBG_ALL                       0xFF

/* different transfer modes */
#define REGISTER                  0x00
#define FIFO                      0x01
#define MODE_2                    0x02    /* mode ?? */ 
#define MODE_3                    0x03
#define MODE_4                    0x04
#define MODE_5                    0x05
#define MODE_6                    0x06
#define MODE_7                    0x07
#define MODE_8                    0x08
#define MODE_9                    0x09
#define X_TABLE                   0x0A
#define Y_TABLE                   0x0B
#define COLOR_TABLE               0x0C
#define CALIBRATION_TABLE         0x0D
#define MODE_E                    0x0E   /* status ?? */
#define MODE_F                    0x0F   /* status ?? */

/* DrvMcuSendRequest */
#define CARRIGE_RETURN            0xC0C0 /* moves lamp back, speed in first two bytes */
#define S_8000                    0x8000
#define S_40B0                    0x40B0
#define S_4000                    0x4000
#define S_2080                    0x2080
#define GO_OFFLINE                0x2040 /* go offline ?*/
#define S_2020                    0x2020
#define S_2000                    0x2000

// color_mode

#define RED_MODE                  2
#define GREEN_MODE                0x0012
#define BLUE_MODE                 0x0014
#define RGB_MODE                  0x0007
#define GRAY_MODE                 3
#define BW_MODE                   0x1107
#define BW_MODE_RED               0x1011
#define BW_MODE_GREEN             0x1012
#define BW_MODE_BLUE              0x1014

#define CALIBRATION               10
#define SCAN                      20

#define LOOP_TIMEOUT              200000    /* timeout-time during loops in us */
#define SCAN_LOOP_TIMEOUT         1000000   /* need to wait much longer for long lines */

#define TIMEOUT                   -1
#define NO_TIMEOUT                0

/* size of scanner area in inch */
#define MAX_WIDTH                 8.5
#define MAX_HEIGHT                11.71      /* guessed */

/* resolution of scanner */
#define X_RESOLUTION  300
#define Y_RESOLUTION  300  



typedef struct {
    double contrast;
    double brightness;
    double gamma;
} color_setting;

    
typedef struct {
    int width;
    int height;
    int resolution;

    double from_top;        /* distance fromm top */
    double from_left;       /* distance from left side */
    
    int num_colors;         /* number of colors, 3 */
    int color_mode;
    color_setting color[3];     /* b/c/g of all colors */
    
    int skip_green;        /* number of lines to wait for green  */
    int skip_blue;         /* number of lines to wait for blue  */

    unsigned char *color_table[3];
//    unsigned char *x_table;
//    unsigned char *y_table;
//    unsigned char *calibration_table;
    
    unsigned char **data;

    int manual_mode;
 
    char *filename;
} scan_image;


/* parallel port probing routine */
extern int port_probe(unsigned short port, int quietflag);
void epp_on(void);
void epp_off(void);

/* low level routines */
int epp_read(unsigned char *data, int num, unsigned char mode);
int epp_read_one(unsigned char *data, unsigned char mode);   

int epp_write(unsigned char *data, int num, unsigned char mode);
int epp_write_one(unsigned char data, unsigned char mode); 

int epp_set_mode(unsigned char mode);
int clear_timeout(void);

inline int check_timer(long long *timer_timer);
inline void init_timer(long long *timer_timer, unsigned long us_time);
void set_port( int );

/* medium level routines */
int read_register(unsigned char* data, int register_nr);
int write_register(unsigned char value, int num);
int power_sequence(int sequence_code);
//int move_carrige_forward(float distance);
//int move_carrige_backward(float distance);
int init_adc(void);


/* high level routines */
//void calc_color_table(unsigned char *table, double contrast, double brightness, double color);
//void calc_x_table(unsigned char *table, float start, int width, int resolution);
//void calc_y_table(unsigned char *table, int height, int resolution);
//void calc_calibration_table(unsigned char *table);

void calc_color_table(scan_image *image);
void calc_x_table(scan_image *image);
void calc_y_table(scan_image *image);
void calc_calibration_table(scan_image *image);

/* these fucntions must be rearenged -- better names */
void calibrate(unsigned char *calibration_table);                 /* not yet */
void scan (scan_image *image, int mode);                     /* this is a brute force approach */
void read_scan_data(scan_image *image, int mode);
//void read_scan_data(const char *filename, int width, int height, int num_colors, int resolution);

/* helper functions */
void calc_contrast(unsigned char *table, int value);
void calc_brightness(unsigned char *table, int value);
void calc_color(unsigned char *table, double value);
void tiff_open(TIFF **tiffile, scan_image *image);
//void tiff_save_line(TIFF *tiffile, unsigned char *rawdata, int position, scan_image *image);
void tiff_close(TIFF *tiffile);
void parse_options(scan_image *image, int argc, char **argv);
void init_image(scan_image **image);




#endif /* pxscan.h */



/* This stuff is for emacs
 * Local variables:
 * c-basic-offset: 4
 * End:       
 */
