/* Register Definition of E3 ASIC */

struct REG0 { // AsicID - R
    unsigned AsicType	:8;
};

struct REG1 { // General Register 0 - R/W
    unsigned 		:8;
};

struct REG2 { // General Register 1 - R/W
    unsigned		:8;
};

struct REG3 { // Command Register 0 - R/W
    unsigned SysReset	:1;
    unsigned FiFoReset	:1;
    unsigned EppUsb	:1;
    unsigned WatchDog	:1;
    unsigned SelfTest	:1;
    unsigned ScanSpeed	:3;
};

struct REG4 { // Command Register 1 - R/W
    unsigned AsicTest		:1;
    unsigned NoLineOffset	:1;
    unsigned Refresh		:1;
    unsigned RefreshForever	:1;
    unsigned WaitDelay		:2;
    unsigned ScanMode		:2;
};

struct REG5 { // Command Register 2 - R/W
    unsigned Sensor	:2;
    unsigned SensorRes	:2;
    unsigned Afe	:3;
    unsigned Adc1210	:1;
};

struct REG6 { // Motor Control - R/W
    unsigned MotorPower	:1;
    unsigned HalfFull	:1;
    unsigned Operation	:3;
    unsigned LineOffset	:3;
};

struct REG7 { // Motor Move Steps High Byte - R/W
    unsigned 	:8;
};

struct REG8 { // Motor Move Steps Low Byte - R/W
    unsigned 	:8;
};

struct REG9 { // Motor Back Steps (Scanning) - R/W
    unsigned 	:8;
};

struct REG10 { // Motor Trigger Period High Byte (Unit: 0.5 us) - R/W
    unsigned 	:8;
};

struct REG11 { // Motor Trigger Period Low Byte (Unit: 0.5 us) - R/W
    unsigned 	:8;
};

struct REG12 { // Sensor Status - R/W
    unsigned MotorMove	:1;
    unsigned MotorStop	:1;
    unsigned HomeSensor	:1;
    unsigned FinishFlag :1;
    unsigned FiFoEmpty	:1;
    unsigned HwSelfTest	:1;
    unsigned ScanStatus	:1;
    unsigned Lamp	:1;
};

struct REG13 { // AFE Programming Register - R/W
    unsigned SDI	:1;
    unsigned SDO	:1;	// data to scanner adc ( = SDI of adc )
    unsigned SCLK	:1;
    unsigned CS		:1;	// = SEN on adc?
    unsigned Reserved	:1;
    unsigned WmVsmp	:3;	// VSMP = Video SAMPle sync pulse, whatever it means here
};

struct REG14 { // Memory Address Port (MSB) - R/W
    unsigned Mem		:5;
    unsigned Reserved		:2;
    unsigned Mem_512_2048	:1;
};

struct REG15 { // Memory Address Port High Byte - R/W
    unsigned	:8;
};

struct REG16 { // Memory Address Port Low Byte - R/W
    unsigned	:8;
};

struct REG17 { // Memory Data Port - R/W
    unsigned 	:8;
};

struct REG18 { // Data Size High - R/W
    unsigned 	:8;
};

struct REG19 { // Data Size Low - R/W
    unsigned 	:8;
};

struct REG20 { // Horizontal Scan Area Start Point High Byte - R/W
    unsigned AreaStart	:6;
    unsigned Reserved	:2;
};

struct REG21 { // Horizontal Scan Area Start Point Low Byte - R/W
    unsigned AreaStart	:8;
};

struct REG22 { // Horizontal Scan Area Width Point High Byte - R/W
    unsigned AreaWidth	:6;
    unsigned Reserved	:2;
};

struct REG23 { // Horizontal Scan Area Start Point Low Byte - R/W
    unsigned AreaWidth	:8;
};

struct REG24 { // Gerneral Purpose IO Register 0 - R
    unsigned ProductID	:5;
    unsigned Gio5	:1;
    unsigned Gio6	:1;
    unsigned Reserved	:1;
};

struct REG25 { // General Purpose IO Register 1 - R/W
    unsigned Gio11	:1;
    unsigned Gio12	:1;
    unsigned Gio13	:1;
    unsigned Gio14	:1;
    unsigned Gio11ctl	:1;
    unsigned Gio12ctl	:1;
    unsigned Gio13ctl	:1;
    unsigned Gio14ctl	:1;
};

struct REG26 { // Buffer Bank R/W Difference Motor Start & Stop - R/W
    unsigned Stop	:4;
    unsigned Start	:4;
};

struct REG27 { // X-Direction & Y-Direction Resolution Control High Byte - R/W
    unsigned YRes		:3;
    unsigned AutoPattern    	:1;
    unsigned XRes		:3;
    unsigned True16Bit		:1;
};

struct REG28 { // X-Direction Resolution Control Low Byte - R/W
    unsigned XRes	:8;
};

struct REG29 { // Y-Direction Resolution Control Low Byte - R/W
    unsigned YRes	:8;
};

struct REG30 { // don't know what it is ... not in the E1 ASIC - should be R/W
               // CDS stands for Correlated Double Sampling, ie the dc-level is
               // supressed.
    unsigned CDS_Data	:4;
    unsigned CDS_Dark	:4;
};

struct REG31 { // Scanner-Buttons ... not in the E1 ASIC - should be R/W
    unsigned Button1	:1;		// Front panel button on D9600. Must be cleared by driver.
    unsigned Button2	:1;
    unsigned Button3	:1;
    unsigned Button4	:1;
    unsigned Button5	:1;
    unsigned Button6	:1;
    unsigned Button7	:1;
    unsigned Button8	:1;
};

// Put all Register structs into unions 

union {
    struct REG0 r;
    unsigned char w :8;
} reg0;

union {
    struct REG1 r;
    unsigned char w :8;
} reg1;

union {
    struct REG2 r;
    unsigned char w;
} reg2;

union {
    struct REG3 r;
    unsigned char w;
} reg3;

union {
    struct REG4 r;
    unsigned char w;
} reg4;

union {
    struct REG5 r;
    unsigned char w;
} reg5;

union {
    struct REG6 r;
    unsigned char w;
} reg6;

union {
    struct REG7 r;
    unsigned char w;
} reg7;

union {
    struct REG8 r;
    unsigned char w;
} reg8;

union {
    struct REG9 r;
    unsigned char w;
} reg9;

union {
    struct REG10 r;
    unsigned char w;
} reg10;

union {
    struct REG11 r;
    unsigned char w;
} reg11;

union {
    struct REG12 r;
    unsigned char w ;
} reg12;

union {
    struct REG13 r;
    unsigned char w;
} reg13;

union {
    struct REG14 r;
    unsigned char w;
} reg14;

union {
    struct REG15 r;
    unsigned char w;
} reg15;

union {
    struct REG16 r;
    unsigned char w;
} reg16;

union {
    struct REG17 r;
    unsigned char w;
} reg17;

union {
    struct REG18 r;
    unsigned char w;
} reg18;

union {
    struct REG19 r;
    unsigned char w;
} reg19;

union {
    struct REG20 r;
    unsigned char w;
} reg20;

union {
    struct REG21 r;
    unsigned char w;
} reg21;

union {
    struct REG22 r;
    unsigned char w;
} reg22;

union {
    struct REG23 r;
    unsigned char w;
} reg23;

union {
    struct REG24 r;
    unsigned char w;
} reg24;

union {
    struct REG25 r;
    unsigned char w;
} reg25;

union {
    struct REG26 r;
    unsigned char w;
} reg26;

union {
    struct REG27 r;
    unsigned char w;
} reg27;

union {
    struct REG28 r;
    unsigned char w;
} reg28;

union {
    struct REG29 r;
    unsigned char w;
} reg29;

union {
    struct REG30 r;
    unsigned char w;
} reg30;

union {
    struct REG31 r;
    unsigned char w;
} reg31;

